/***************************************************************************/
/*                                                                         */
/*  internal.h                                                             */
/*                                                                         */
/*    Internal header files (specification only).                          */
/*                                                                         */
/*  Copyright 1996-2001 by                                                 */
/*  David Turner, Robert Wilhelm, and Werner Lemberg.                      */
/*                                                                         */
/*  This file is part of the FreeType project, and may only be used,       */
/*  modified, and distributed under the terms of the FreeType project      */
/*  license, LICENSE.TXT.  By continuing to use, modify, or distribute     */
/*  this file you indicate that you have read the license and              */
/*  understand and accept it fully.                                        */
/*                                                                         */
/***************************************************************************/


  /*************************************************************************/
  /*                                                                       */
  /* This file is automatically included by `ft2build.h'.                  */
  /* Do not include it manually!                                           */
  /*                                                                       */
  /*************************************************************************/


#define FT_INTERNAL_OBJECTS_H           <freetype/internal/ftobjs.h>
#define FT_INTERNAL_STREAM_H            <freetype/internal/ftstream.h>
#define FT_INTERNAL_MEMORY_H            <freetype/internal/ftmemory.h>
#define FT_INTERNAL_EXTENSION_H         <freetype/internal/ftextend.h>
#define FT_INTERNAL_DEBUG_H             <freetype/internal/ftdebug.h>
#define FT_INTERNAL_CALC_H              <freetype/internal/ftcalc.h>
#define FT_INTERNAL_DRIVER_H            <freetype/internal/ftdriver.h>
#define FT_INTERNAL_EXTEND_H            <freetype/internal/ftextend.h>

#define FT_INTERNAL_SFNT_H              <freetype/internal/sfnt.h>

#define FT_INTERNAL_TRUETYPE_TYPES_H    <freetype/internal/tttypes.h>
#define FT_INTERNAL_TYPE1_TYPES_H       <freetype/internal/t1types.h>
#define FT_INTERNAL_CFF_TYPES_H         <freetype/internal/cfftypes.h>
#define FT_INTERNAL_FNT_TYPES_H         <freetype/internal/fnttypes.h>

#define FT_INTERNAL_POSTSCRIPT_NAMES_H  <freetype/internal/psnames.h>
#define FT_INTERNAL_POSTSCRIPT_AUX_H    <freetype/internal/psaux.h>

#define FT_INTERNAL_AUTOHINT_H          <freetype/internal/autohint.h>


/* END */
